@@ -4,29 +4,39 @@
 Initialize with a MSAServiceDefintion Instance to control the features and functions of the MSAApp.
 
 """
+import json
 import os
 from asyncio import Task
 from datetime import datetime
+from functools import wraps
 from typing import Any, Dict, List, Optional, Type, Union
 
+from apscheduler.schedulers.asyncio import AsyncIOScheduler
+from apscheduler.schedulers.background import BackgroundScheduler
 from dapr.clients import DaprClient
+from dapr.ext.fastapi import DaprApp
 from fastapi import FastAPI, HTTPException
 from fastapi.encoders import jsonable_encoder
 from fastapi.exception_handlers import http_exception_handler
 from fastapi.exceptions import RequestValidationError
 from fastapi.responses import ORJSONResponse
+from fs.base import FS
 from loguru import logger as logger_gruru
-from msaBase.config import MSAServiceDefinition, MSAServiceStatus
+from msaBase.config import ConfigDTO, ConfigInput, MSAServiceDefinition, MSAServiceStatus
 from msaBase.errorhandling import getMSABaseExceptionHandler
 from msaBase.logger import init_logging
 from msaBase.models.functionality import FunctionalityTypes
 from msaBase.models.middlewares import MiddlewareTypes
 from msaBase.models.sysinfo import MSASystemGPUInfo, MSASystemInfo
 from msaBase.sysinfo import get_sysgpuinfo, get_sysinfo
+from msaBase.utils.constants import PUBSUB_NAME, REGISTRY_TOPIC, SERVICE_TOPIC
+from msaDocModels import health
 from msaDocModels.health import MSAHealthDefinition, MSAHealthMessage
 from msaDocModels.openapi import MSAOpenAPIInfo
 from msaDocModels.scheduler import MSASchedulerStatus, MSASchedulerTaskDetail, MSASchedulerTaskStatus
 from msaDocModels.sdu import SDUVersion
+from msaFilesystem.msafs import MSAFilesystem
+from slowapi import Limiter
 from starlette import status
 from starlette.exceptions import HTTPException as StarletteHTTPException
 from starlette.requests import Request
@@ -84,16 +94,19 @@ class MSAApp(FastAPI):
 
     Note:
         As with FastApi the MSAApp provides two events:
-        ``startup``: A list of callables to run on application startup. Startup handler callables do not take any arguments, and may be be either standard functions, or async functions.
-        ``shutdown``: A list of callables to run on application shutdown. Shutdown handler callables do not take any arguments, and may be be either standard functions, or async functions.
+        ``startup``: A list of callables to run on application startup. Startup handler callables do not take
+        any arguments, and may be be either standard functions, or async functions.
+        ``shutdown``: A list of callables to run on application shutdown. Shutdown handler callables do not
+        take any arguments, and may be be either standard functions, or async functions.
         Those are also used internally, which are triggered before the external events.
 
         Do not include the `self` parameter in the ``Args`` section.
 
     Args:
         settings: MSAServiceDefinition (Must be provided), instance of a service definition with all settings
-        sql_models: List of SQLModel Default None, provide list of your SQLModel Classes and the instance can create CRUD API and if site is enabled also UI for CRUD
-        auto_mount_site: Default True, if site is enabled in settings and this is true, mounts the site in internal startup event.
+        sql_models: List of SQLModel Default None, provide list of your SQLModel Classes and the instance can create
+        CRUD API and if site is enabled also UI for CRUD auto_mount_site: Default True,
+        if site is enabled in settings and this is true, mounts the site in internal startup event.
 
     Attributes:
         logger: loguru logger instance
@@ -110,11 +123,12 @@ class MSAApp(FastAPI):
     def __init__(
         self,
         settings: MSAServiceDefinition,
-        auto_mount_site: bool = True,
-        title: str = "FastAPI",
-        description: str = "",
-        version: str = "0.1.0",
-        openapi_url: Optional[str] = "/openapi.json",
+        auto_mount_site: Optional[bool] = True,
+        title: Optional[str] = None,
+        description: Optional[str] = None,
+        host: Optional[str] = None,
+        version: Optional[str] = None,
+        openapi_url: Optional[str] = None,
         openapi_tags: Optional[List[Dict[str, Any]]] = None,
         terms_of_service: Optional[str] = None,
         contact: Optional[Dict[str, Union[str, Any]]] = None,
@@ -124,18 +138,23 @@ class MSAApp(FastAPI):
     ) -> None:
         # call super class __init__
         super().__init__(*args, **settings.fastapi_kwargs)
+        self.settings = settings
 
+        self.previous_settings = None
+        self.one_time_config = False
         self.logger = logger_gruru
         self.fastApi = FastAPI
-        self.title = title
-        self.description = description
-        self.version = version
-        self.openapi_url = openapi_url
+        self.daprApp = DaprApp(self)
+        self.title = title if title else self.settings.title
+        self.description = description if description else self.settings.description
+        self.host = host if host else self.settings.host
+
+        self.version = version if version else self.settings.version
+        self.openapi_url = openapi_url if openapi_url else self.settings.openapi_url
         self.auto_mount_site: bool = auto_mount_site
-        self.settings = settings
         self.SDUVersion = SDUVersion(version=self.settings.version, creation_date=datetime.utcnow().isoformat())
         self.license_info = license_info
-        self.contact = contact
+        self.contact = contact if contact else self.settings.contact
         self.terms_of_service = terms_of_service
         self.openapi_tags = openapi_tags
         self.healthdefinition: MSAHealthDefinition = self.settings.healthdefinition
@@ -153,15 +172,70 @@ class MSAApp(FastAPI):
         init_logging()
         self.add_middlewares()
         self.add_functionality()
-        self.logger.info("Events - Add Internal Handlers")
         self.add_event_handler("shutdown", self.shutdown_event)
         self.add_event_handler("startup", self.startup_event)
+        self.create_dapr_endpoint()
 
-    def logger_info(self, message: str, topic_name: str = ""):
+    def create_dapr_endpoint(self):
+        """
+        Subscribes service to pubsub topic through which new configs will be received.
+        """
+
+        @self.daprApp.subscribe(pubsub=PUBSUB_NAME, topic=SERVICE_TOPIC)
+        async def read_config(received_config: ConfigInput) -> None:
+            """
+            Receives new config and updates current settings with received data.
+
+            Parameters:
+                received_config: Data to update current config with.
+            """
+            try:
+                self.logger.info(f"Received config from spkRegistry. Data: {received_config.data}")
+                if received_config.data.config.name == self.settings.name:
+                    reload_needed = self.update_settings(received_config.data.config, received_config.data.one_time)
+                    if reload_needed:
+                        self.logger.info("New config needs reload.")
+                        with open("config.json", "w") as json_file:
+                            json.dump(received_config.data.dict(), json_file)
+
+                        self.logger.info("New config saved to config.json")
+
+            except Exception as ex:
+                self.logger.info(ex)
+
+    def uses_temporary_config(function):
+        """
+        Makes an endpoint use one-time config whenever it is present.
+
+        Parameters:
+            function: an endpoint to wrap
+        """
+
+        @wraps(function)
+        def decorator(self, *args, **kwargs):
+            result = function(self, *args, **kwargs)
+            if self.one_time_config:
+                self.logger.info("One-time config used. Loading previous config...")
+                self.update_settings(self.previous_settings)
+
+                self.previous_settings = None
+                self.one_time_config = False
+            return result
+
+        return decorator
+
+    def logger_info(self, message: str, topic_name: str = "") -> None:
+        """
+        Sends message to pubsub topic.
+
+        Parameters:
+            message: JSON message to send.
+            topic_name: name of pubsub topic that needs this message.
+        """
         if topic_name:
             with DaprClient() as client:
                 client.publish_event(
-                    pubsub_name="spkpubsub",
+                    pubsub_name=PUBSUB_NAME,
                     topic_name=topic_name,
                     data=message,
                     data_content_type="application/json",
@@ -318,13 +392,11 @@ class MSAApp(FastAPI):
         """
         Get Service OpenAPI Schema
 
-        Args:
+        Parameters:
             request: The input http request object
 
         Returns:
-            openapi: ORJSONResponse openapi schema
-
-
+            ORJSONResponse openapi schema
         """
         self.logger.info("Called - get_services_openapi_schema :" + str(request.url))
 
@@ -344,15 +416,15 @@ class MSAApp(FastAPI):
 
     async def msa_exception_handler(self, request: Request, exc: HTTPException) -> Response:
         """
-        Handles all HTTPExceptions if enabled with HTML Response or forward error if the code is in the exclude settings list.
+        Handles all HTTPExceptions if enabled with HTML
+        Response or forward error if the code is in the exclude settings list.
 
-        Args:
+        Parameters:
             request: The input http request object
             exc : The HTTPException instance
 
         Returns:
-            HTTPException
-
+            HTTPResponse: response with status code corresponding to the handled exception.
         """
         error_content = {
             "request": request.__dict__,
@@ -364,10 +436,11 @@ class MSAApp(FastAPI):
         return await http_exception_handler(request, exc)
 
     def get_sduversion(self) -> SDUVersion:
-        """Get SDUVersion
+        """
+        Get SDUVersion
+
         Returns:
             sdu_version: Pydantic Version Info Model.
-
         """
         return self.SDUVersion
 
@@ -396,6 +469,15 @@ class MSAApp(FastAPI):
         return oai
 
     async def validation_exception_handler(self, request: Request, exc: RequestValidationError) -> JSONResponse:
+        """
+        Handles validation error exception and returns exception info as a JSON.
+
+        Parameters:
+            request: Request that raised the exception
+            exc: exception to handle
+        Returns:
+            response: JSONResponse that represents the exception
+        """
         self.logger.error("validation_exception_handler - " + str(exc.errors()))
         return JSONResponse(
             status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
@@ -457,6 +539,47 @@ class MSAApp(FastAPI):
             except Exception as ex:
                 getMSABaseExceptionHandler().handle(ex)
 
+    def update_settings(self, new_config: MSAServiceDefinition, one_time=False) -> bool:
+        """
+        Updates app configuration.
+
+        Parameters:
+            new_config: MSAServiceDefinition. Config received from SPKRegistry.
+            one_time: a flag for using the config only one time.
+        Returns:
+            bool. True if app reload is needed, False otherwise.
+        """
+        if one_time:
+            self.previous_settings = self.settings
+
+        for middleware in MiddlewareTypes:
+            current_middleware = getattr(self.settings, middleware.name, None)
+
+            new_middleware = getattr(new_config, middleware.name, None)
+
+            if (current_middleware is not None and new_middleware is not None) and (
+                current_middleware != new_middleware
+            ):
+                return True
+
+        for functionality in FunctionalityTypes:
+            current_functionality = getattr(self.settings, functionality.name, None)
+
+            new_functionality = getattr(new_config, functionality.name, None)
+            reload_needed = functionality.need_restart
+
+            if (current_functionality is not None and new_functionality is not None) and (
+                current_functionality != new_functionality
+            ):
+
+                if reload_needed:
+                    return True
+
+                setattr(self.settings, functionality.name, new_functionality)
+                self.choose_functionality_configurator(functionality)()
+
+        return False
+
     def unknown_middleware(self) -> None:
         """
         Unknown Middleware, doing nothing
@@ -477,7 +600,6 @@ class MSAApp(FastAPI):
 
         Returns:
             func: The configurator.
-
         """
         configurator_mappings = {
             MiddlewareTypes.profiler: self.configure_profiler_middleware,
@@ -502,7 +624,6 @@ class MSAApp(FastAPI):
 
         Returns:
             func: The configurator.
-
         """
         configurator_mappings = {
             FunctionalityTypes.uvloop: self.configure_event_loop,
@@ -534,7 +655,12 @@ class MSAApp(FastAPI):
             )
         self.add_api_route("/", self.get_sduversion, tags=["service"], response_model=SDUVersion)
         self.add_api_route("/sysinfo", get_sysinfo, tags=["service"], response_model=MSASystemInfo)
-        self.add_api_route("/sysgpuinfo", self.get_system_gpu_info, tags=["service"], response_model=MSASystemGPUInfo)
+        self.add_api_route(
+            "/sysgpuinfo",
+            self.get_system_gpu_info,
+            tags=["service"],
+            response_model=MSASystemGPUInfo,
+        )
         self.add_api_route("/settings", self.get_services_settings, tags=["service"])
         self.add_api_route(
             "/status",
@@ -587,6 +713,7 @@ class MSAApp(FastAPI):
         self.add_exception_handler(RequestValidationError, self.validation_exception_handler)
 
     def configure_healthdefinition(self) -> None:
+        """Configure health definition and start healthcheck thread."""
         self.logger.info("Init Healthcheck")
         from msaBase.healthcheck import MSAHealthCheck
 
@@ -609,8 +736,15 @@ class MSAApp(FastAPI):
         self.logger.info("Enable Abstract Filesystem")
         from msaFilesystem.msafs import MSAFilesystem
 
-        self.abstract_fs = MSAFilesystem(fs_url=self.settings.abstract_fs_url)
-        self.fs = self.abstract_fs.fs
+        if self.settings.abstract_fs:
+            try:
+                self.logger.info("Closing Abstract Filesystem")
+                self.fs.close()
+            except Exception as ex:
+                getMSABaseExceptionHandler().handle(ex, "Error: Closing Abstract Filesystem failed:")
+        else:
+            self.fs = self.abstract_fs.fs
+            self.abstract_fs = MSAFilesystem(fs_url=self.settings.abstract_fs_url)
 
     def configure_msgpack_middleware(self) -> None:
         """Add Middleware MSGPack"""
@@ -660,16 +794,22 @@ class MSAApp(FastAPI):
         self.logger.info("Add Background Scheduler")
         from apscheduler.schedulers.background import BackgroundScheduler
 
-        self.background_scheduler = BackgroundScheduler()
-        self.background_scheduler.start()
+        if self.background_scheduler and self.background_scheduler.get_jobs():
+            self.background_scheduler.shutdown()
+        else:
+            self.background_scheduler = BackgroundScheduler()
+            self.background_scheduler.start()
 
     def configure_asyncio_scheduler(self) -> None:
         """Add Asyncio Scheduler"""
         self.logger.info("Add Asyncio Scheduler")
         from apscheduler.schedulers.asyncio import AsyncIOScheduler
 
-        self.asyncio_scheduler = AsyncIOScheduler()
-        self.asyncio_scheduler.start()
+        if self.asyncio_scheduler and self.asyncio_scheduler.get_jobs():
+            self.asyncio_scheduler.shutdown()
+        else:
+            self.asyncio_scheduler = AsyncIOScheduler()
+            self.asyncio_scheduler.start()
 
     def configure_event_loop(self) -> None:
         """Enable UVLoop"""
@@ -679,7 +819,7 @@ class MSAApp(FastAPI):
         uvloop.install()
 
     def configure_cors_middleware(self) -> None:
-        """ "Add Middleware CORS"""
+        """Add Middleware CORS"""
         self.logger.info("Add Middleware CORS")
         from starlette.middleware.cors import CORSMiddleware
 
@@ -711,3 +851,17 @@ class MSAApp(FastAPI):
 
         self.logger.info("Add Middleware HTTPSRedirect")
         self.add_middleware(HTTPSRedirectMiddleware)
+
+    def send_config(self) -> None:
+        """
+        Sends current config to pubsub registry topic.
+        """
+        try:
+            self.logger.info("Start send config to pubsub")
+            with open("config.json") as json_file:
+                config = MSAServiceDefinition.parse_obj(json.load(json_file))
+                data = ConfigDTO(config=config, one_time=False)
+            self.logger_info(data.json(), REGISTRY_TOPIC)
+            self.logger.info(f"Sent config to pubsub, {data}")
+        except Exception as ex:
+            self.logger.error(f"An error occurred while trying to send config to spkRegistry. Exception: {ex}")
